#ifndef __EDG_WORKLOAD_LOGGING_EVENT__
#define __EDG_WORKLOAD_LOGGING_EVENT__

#include "edg/workload/logging/client/CountRef.h"
#include "edg/workload/common/jobid/JobId.h"

#include <utility>
#include <vector>
#include <string>

#ident "$Header: /local/reps/lcgware/workload/logging/client/Event.h.T,v 1.29 2003/04/11 05:59:12 salvet Exp $"

/** @file Event.h
 *  @version $Revision: 1.29 $
 */

/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ./logging/client/Event.h.T

*/

#include "edg/workload/logging/client/events.h"

EWL_BEGIN_NAMESPACE;

class Event {
	friend class Job;
	friend class ServerConnection;
	friend class CountRef<Event>;
public:
	/** Event type codes.
	 * Identify which of the event fields are valid.
	 */

	enum Type {
		UNDEF = 0,
		TRANSFER,	/**< Start, success, or failure of job transfer to another component */
		ACCEPTED,	/**< Accepting job (successful couterpart to Transfer) */
		REFUSED,	/**< Refusing job (unsuccessful couterpart to Transfer) */
		ENQUEUED,	/**< The job has been enqueued in an inter-component queue */
		DEQUEUED,	/**< The job has been dequeued from an inter-component queue */
		HELPERCALL,	/**< Helper component is called */
		HELPERRETURN,	/**< Helper component is returning the control */
		RUNNING,	/**< Executable started */
		RESUBMISSION,	/**< Result of resubmission decision */
		DONE,	/**< Execution terminated (normally or abnormally) */
		CANCEL,	/**< Cancel operation has been attempted on the job */
		ABORT,	/**< Job aborted by system */
		CLEAR,	/**< Job cleared, output sandbox removed */
		PURGE,	/**< Job is purged from bookkepping server */
		MATCH,	/**< Matching CE found */
		PENDING,	/**< No match found yet */
		REGJOB,	/**< New job registration */
		CHKPT,	/**< Application-specific checkpoint record */
		LISTENER,	/**< Listening network port for interactive control */
		CURDESCR,	/**< current state of job processing (optional event) */
		USERTAG,	/**< user tag -- arbitrary name=value pair */
		CHANGEACL,	/**< Management of ACL stored on bookkepping server */
		TYPE_MAX
	};

	/** Event attribute symbolic identifier. */
	enum Attr {
	/**
	 * Chkpt: checkpoint value
	 */
		CLASSAD,
	/**
	 * CurDescr: description of current job transformation (output of helper)
	 */
		DESCR,
	/**
	 * Transfer: destination hostname
	 */
		DEST_HOST,
	/**
	 * Match: Id of the destination CE/queue
	 */
		DEST_ID,
	/**
	 * Transfer: destination instance
	 */
		DEST_INSTANCE,
	/**
	 * Transfer: destination internal jobid
	 */
		DEST_JOBID,
	/**
	 * Transfer: destination where the job is being transfered to
	 */
		DESTINATION,
	/**
	 * Done: process exit code
	 */
		EXIT_CODE,
	/**
	 * Accepted: where was the job received from
	 * Refused: where was the job received from
	 */
		FROM,
	/**
	 * Accepted: sending component hostname
	 * Refused: sending component hostname
	 */
		FROM_HOST,
	/**
	 * Accepted: sending component instance
	 * Refused: sending component instance
	 */
		FROM_INSTANCE,
	/**
	 * HelperCall: name of the called component
	 * HelperReturn: name of the called component
	 */
		HELPER_NAME,
	/**
	 * HelperCall: parameters of the call
	 */
		HELPER_PARAMS,
	/**
	 * common: hostname of the machine where the event was generated
	 */
		HOST,
	/**
	 * RegJob: job description
	 */
		JDL,
	/**
	 * EnQueued: job description in receiver language
	 * Transfer: job description in receiver language
	 */
		JOB,
	/**
	 * common: DataGrid job id of the source job
	 */
		JOBID,
	/**
	 * RegJob: job type
	 */
		JOBTYPE,
	/**
	 * common: logging level (system, debug, ...)
	 */
		LEVEL,
	/**
	 * Accepted: new jobId (Condor, Globus ...) assigned by the receiving component
	 * DeQueued: new jobId assigned by the receiving component
	 */
		LOCAL_JOBID,
	/**
	 * UserTag: tag name
	 */
		NAME,
	/**
	 * Running: worker node where the executable is run
	 */
		NODE,
	/**
	 * RegJob: NetworkServer handling the job
	 */
		NS,
	/**
	 * RegJob: number of subjobs
	 */
		NSUBJOBS,
	/**
	 * ChangeACL: operation requested to perform with ACL (add, remove)
	 */
		OPERATION,
	/**
	 * RegJob: jobid of parent job
	 */
		PARENT,
	/**
	 * ChangeACL: ACL permission to change (currently only READ)
	 */
		PERMISSION,
	/**
	 * ChangeACL: type of permission requested ('allow', 'deny')
	 */
		PERMISSION_TYPE,
	/**
	 * common: message priority (yet 0 for asynchronous and 1 for synchronous transfers)
	 */
		PRIORITY,
	/**
	 * DeQueued: queue name
	 * EnQueued: destination queue
	 */
		QUEUE,
	/**
	 * Abort: reason of abort
	 * Cancel: detailed description
	 * Clear: why the job was cleared
	 * Done: reason for the change
	 * EnQueued: detailed description of transfer, especially reason of failure
	 * Pending: why matching CE cannot be found
	 * Refused: reason of refusal
	 * Resubmission: reason for the decision
	 * Transfer: detailed description of transfer, especially reason of failure
	 */
		REASON,
	/**
	 * EnQueued: result of the attempt
	 * Resubmission: result code
	 * Transfer: result of the attempt
	 */
		RESULT,
	/**
	 * HelperReturn: returned data
	 */
		RETVAL,
	/**
	 * RegJob: seed for subjob id generation
	 */
		SEED,
	/**
	 * common: sequence code assigned to the event
	 */
		SEQCODE,
	/**
	 * common: source (WMS component) which generated this event
	 */
		SOURCE,
	/**
	 * common: instance of WMS component (e.g. service communication endpoint)
	 */
		SRC_INSTANCE,
	/**
	 * HelperCall: whether the logging component is called or calling one
	 * HelperReturn: whether the logging component is called or calling one
	 */
		SRC_ROLE,
	/**
	 * Cancel: classification of the cancel
	 * Done: way of termination
	 */
		STATUS_CODE,
	/**
	 * Listener: hostname
	 */
		SVC_HOST,
	/**
	 * Listener: port instance name
	 */
		SVC_NAME,
	/**
	 * Listener: port number
	 */
		SVC_PORT,
	/**
	 * Chkpt: checkpoint tag
	 * Resubmission: value of the attribute on which the decision is based
	 */
		TAG,
	/**
	 * common: timestamp of event generation
	 */
		TIMESTAMP,
	/**
	 * common: identity (cert. subj.) of the generator
	 */
		USER,
	/**
	 * ChangeACL: DN or VOMS parameter (in format VO:group)
	 */
		USER_ID,
	/**
	 * ChangeACL: type of information given in user_id (DN or VOMS)
	 */
		USER_ID_TYPE,
	/**
	 * UserTag: tag value
	 */
		VALUE,
		ATTR_MAX
	};

	enum ClearCode {
		CLEAR_USER,	/**< user retrieved output sandbox */
		CLEAR_TIMEOUT,	/**< timed out, resource purge forced */
		CLEAR_NOOUTPUT,	/**< no output was generated */
	};
	enum RegJobCode {
		REGJOB_SIMPLE,	/**< simple job */
		REGJOB_DAG,	/**< dag (containing static set of subjobs) */
		REGJOB_PARTITIONABLE,	/**< partitionable (may become partitioned) */
		REGJOB_PARTITIONED,	/**< partitioned (dynamically created dag) */
	};
	enum EnQueuedCode {
		ENQUEUED_START,	/**< the sending component has started or is about to start the transfer */
		ENQUEUED_OK,	/**< job was sent successfully */
		ENQUEUED_REFUSED,	/**< job was refused by the other component */
		ENQUEUED_FAIL,	/**< transfer failed for other reason than explicit refusal (eg. network timeout) */
	};
	enum ResubmissionCode {
		RESUBMISSION_WILLRESUB,	/**< will be resubmitted */
		RESUBMISSION_WONTRESUB,	/**< will not be resubmitted */
	};
	enum TransferCode {
		TRANSFER_START,	/**< the sending component has started or is about to start the transfer */
		TRANSFER_OK,	/**< job was sent successfully */
		TRANSFER_REFUSED,	/**< job was refused by the other component */
		TRANSFER_FAIL,	/**< transfer failed for other reason than explicit refusal (eg. network timeout) */
	};
	enum DoneCode {
		DONE_OK,	/**< terminated by itself */
		DONE_FAILED,	/**< disappeared from LRMS */
		DONE_CANCELLED,	/**< cancelled by user request */
	};
	enum CancelCode {
		CANCEL_REQ,	/**< request acknowledged */
		CANCEL_REFUSE,	/**< request declined by this component */
		CANCEL_DONE,	/**< request completed by whole WMS */
		CANCEL_ABORT,	/**< request refused by whole WMS */
	};
	enum HelperCallCode {
		HELPERCALL_CALLING,	/**< the logging component is caller */
		HELPERCALL_CALLED,	/**< the logging component is callee */
	};
	enum HelperReturnCode {
		HELPERRETURN_CALLING,	/**< the logging component is caller */
		HELPERRETURN_CALLED,	/**< the logging component is callee */
	};

	enum AttrType { INT_T, STRING_T, TIMEVAL_T, PORT_T, LOGSRC_T, JOBID_T };

	Type 	type;

	Event(void);
        Event(edg_wll_Event *);
	Event(const Event &);
	~Event(void);


	/** Assign new Event to an existing instance. */
	Event & operator= (const Event &);

	/** String representation of the event type */
	const std::string & name(void) const;

	/** Retrieve integer attribute */
	int	getValInt(Attr) const;

	/** Retrieve string attribute */
	std::string getValString(Attr) const;

        /** Retrieve time attribute */
        struct timeval	getValTime(Attr) const;
		
        /** Retrieve jobid attribute */
        const edg::workload::common::jobid::JobId getValJobId(Attr) const;

	/** Attribute name */
	const std::string & getAttrName(Attr) const;

	/** List of attributes and types valid for this instance */
	const std::vector<std::pair<Attr,AttrType> >  &	getAttrs(void) const;

private:
	static void	destroyFlesh(void *);
	CountRef<Event>	*flesh;
};

EWL_END_NAMESPACE;

#endif
