#ifndef __EDG_WORKLOAD_LOGGING_JOBSTATUS__
#define __EDG_WORKLOAD_LOGGING_JOBSTATUS__

/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ./logging/client/JobStatus.h.T

*/


#include "edg/workload/logging/client/CountRef.h"
#include "edg/workload/common/jobid/JobId.h"

#include "edg/workload/common/jobid/jobid.h"
#include <sys/time.h>
#include "edg/workload/logging/client/jobstat.h"

#include <string>
#include <utility>
#include <vector>

EWL_BEGIN_NAMESPACE;

/**
 * Description of job status.
 * The status is computed from a sequence of logged events
 */


class JobStatus {
	friend class Job;
	friend class CountRef<JobStatus>;
public:
	enum Code {
		UNDEF = 0,	/**< indicates invalid, i.e. uninitialized instance */
		SUBMITTED,	/**< entered by the user to the User Interface or registered by Job Partitioner */
		WAITING,	/**< Accepted by WMS, waiting for resource allocation */
		READY,	/**< Matching resources found */
		SCHEDULED,	/**< Accepted by LRMS queue */
		RUNNING,	/**< Executable is running */
		DONE,	/**< Execution finished, output is available */
		CLEARED,	/**< Output transfered back to user and freed */
		ABORTED,	/**< Aborted by system (at any stage) */
		CANCELLED,	/**< Cancelled by user */
		UNKNOWN,	/**< Status cannot be determined */
		PURGED,	/**< Job has been purged from bookkeeping server (for LB->RGMA interface) */
		CODE_MAX
	};

	enum Attr {
	/** ACL of the job */
		ACL,
	/** Reason of cancel */
		CANCEL_REASON,
	/** Cancellation request in progress */
		CANCELLING,
	/** Worker node where the job is executed */
		CE_NODE,
	/** list of subjob IDs */
		CHILDREN,
	/** summary (histogram) of children job states */
		CHILDREN_HIST,
	/** number of subjobs */
		CHILDREN_NUM,
	/** full status information of the children */
		CHILDREN_STATES,
	/** Id within Condor-G */
		CONDOR_ID,
	/** ClassAd passed to Condor-G for last job execution */
		CONDOR_JDL,
	/** Consumed CPU time */
		CPU_TIME,
	/** ID of CE where the job is being sent */
		DESTINATION,
	/** Return code */
		DONE_CODE,
	/** Unix exit code */
		EXIT_CODE,
	/** Sources of the missing information */
		EXPECT_FROM,
	/** Some logged information has not arrived yet */
		EXPECT_UPDATE,
	/** Globus allocated Id */
		GLOBUS_ID,
	/** User submitted job description */
		JDL,
	/** Id of the job */
		JOB_ID,
	/** Type of job */
		JOBTYPE,
	/** Last known event of the job */
		LAST_UPDATE_TIME,
	/** Id within LRMS */
		LOCAL_ID,
	/** Where the job is being processed */
		LOCATION,
	/** Full job description after matchmaking */
		MATCHED_JDL,
	/** Network server handling the job */
		NETWORK_SERVER,
	/** Job owner */
		OWNER,
	/** parent job of subjob */
		PARENT_JOB,
	/** Reason of being in this status, if any */
		REASON,
	/** The job was resubmitted */
		RESUBMITTED,
	/** Job RSL sent to Globus */
		RSL,
	/** string used for generation of subjob IDs */
		SEED,
	/** When entered this status */
		STATE_ENTER_TIME,
	/** When all previous states were entered */
		STATE_ENTER_TIMES,
	/** Subjob failed (the parent job will fail too) */
		SUBJOB_FAILED,
	/** List of pairs (user_tag, user_value) */
		USER_TAGS,
		ATTR_MAX
	};

	enum {
		DONE_CODE_OK,	/**< Finished correctly */
		DONE_CODE_FAILED,	/**< Execution failed */
		DONE_CODE_CANCELLED,	/**< Cancelled by user */
	};
	enum {
		JOBTYPE_SIMPLE,	/**< simple job */
		JOBTYPE_DAG,	/**< composite job */
	};
	enum AttrType { INT_T, 
			STRING_T, 
			TIMEVAL_T, 
			BOOL_T,
			JOBID_T,
			INTLIST_T, 
			STRLIST_T, 
			TAGLIST_T, 
			STSLIST_T 
	};
  
	/** Numeric status code */
	Code	status;
  
	/** String representation of the status code */
	const std::string & name(void) const;
  
	/** Retrieve integer attribute */
	int	getValInt(Attr) const;
  
	/** Retrieve string attribute */
	std::string getValString(Attr) const;
  
	/** Retrieve time attribute */
	struct timeval	getValTime(Attr) const;
  
	/** Retrieve jobid attribute */
	const edg::workload::common::jobid::JobId  getValJobId(Attr) const;

	/** Retrieve bool attribute */
	bool getValBool(Attr) const;

	/** Retrieve int list attribute */
	const std::vector<int> getValIntList(Attr) const;

	/** Retrieve string list attribute */
	const std::vector<std::string> getValStringList(Attr) const;

	/** Retrieve tag list attribute */
	const std::vector<std::pair<std::string,std::string> > getValTagList(Attr) const;

	/** Retrieve job status list attribute */
	const std::vector<JobStatus> getValJobStatusList(Attr) const;

	/** Attribute name */
	const std::string& getAttrName(Attr) const;
  
	/** List of attributes and types valid for this instance */
	const std::vector<std::pair<Attr,AttrType> >& getAttrs(void) const;
  
	JobStatus(void);
	JobStatus(const JobStatus &);
	JobStatus & operator=(const JobStatus &);
	JobStatus(const edg_wll_JobStat &);
	JobStatus & operator=(const edg_wll_JobStat&);
	virtual ~JobStatus();

protected:
	edg_wll_JobStat *c_ptr(void);

private:
	static void	destroyFlesh(void *);
	CountRef<JobStatus> *flesh;
};

EWL_END_NAMESPACE;

#endif

